###################################################################################
# ocr_translate - a django app to perform OCR and translation of images.          #
# Copyright (C) 2023-present Davide Grassano                                      #
#                                                                                 #
# This program is free software: you can redistribute it and/or modify            #
# it under the terms of the GNU General Public License as published by            #
# the Free Software Foundation, either version 3 of the License.                  #
#                                                                                 #
# This program is distributed in the hope that it will be useful,                 #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                  #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                   #
# GNU General Public License for more details.                                    #
#                                                                                 #
# You should have received a copy of the GNU General Public License               #
# along with this program.  If not, see {http://www.gnu.org/licenses/}.           #
#                                                                                 #
# Home: https://github.com/Crivella/ocr_translate                                 #
###################################################################################
"""Initialize the server based on environment variables."""
import json
import logging
from importlib.metadata import entry_points
from pathlib import Path

from django.db.models import Count

from .. import models as m
from .box import load_box_model
from .lang import load_lang_dst, load_lang_src
from .ocr import load_ocr_model
from .tsl import load_tsl_model

logger = logging.getLogger('ocr.general')

def init_most_used():
    """Initialize the server with the most used languages and models."""
    src = m.Language.objects.annotate(count=Count('trans_src')).order_by('-count').first()
    dst = m.Language.objects.annotate(count=Count('trans_dst')).order_by('-count').first()

    if src:
        load_lang_src(src.iso1)
    if dst:
        load_lang_dst(dst.iso1)

    box = m.OCRBoxModel.objects.annotate(count=Count('box_runs')).order_by('-count').first()
    ocr = m.OCRModel.objects.annotate(count=Count('ocr_runs')).order_by('-count').first()
    tsl = m.TSLModel.objects.annotate(count=Count('tsl_runs')).order_by('-count').first()

    if box:
        load_box_model(box.name)
    if ocr:
        load_ocr_model(ocr.name)
    if tsl:
        load_tsl_model(tsl.name)

def auto_create_languages():
    """Create Language objects from json file."""
    cwd = Path(__file__).parent
    with open(cwd / 'languages.json', encoding='utf-8') as f:
        langs = json.load(f)

    for lang in langs:
        logger.debug(f'Creating language: {lang}')
        name = lang.pop('name')
        iso1 = lang.pop('iso1')
        iso2t = lang.pop('iso2t')
        iso2b = lang.pop('iso2b')
        iso3 = lang.pop('iso3')
        def_opt = lang.pop('default_options', {})
        opt_obj, _ = m.OptionDict.objects.get_or_create(options=def_opt)
        l, _ = m.Language.objects.get_or_create(name=name, iso1=iso1, iso2t=iso2t, iso2b=iso2b, iso3=iso3)
        l.default_options = opt_obj
        # for k,v in lang.items():
        #     setattr(l, k, v)
        l.save()

def load_ept_data(namespace):
    """Load all entrypoints from a namespace into a list"""
    res = []

    for ept in entry_points(group=namespace):
        # Copy required for pop on dct
        res.append(ept.load().copy())

    return res

def auto_create_box():
    """Create OCRBoxModel objects from entrypoints."""
    for box in load_ept_data('ocr_translate.box_data'):
        logger.debug(f'Creating box model: {box}')
        lang = box.pop('lang')
        lcode = box.pop('lang_code')
        entrypoint = box.pop('entrypoint')
        iso1_map = box.pop('iso1_map', {})
        def_opt = box.pop('default_options', {})
        opt_obj, _ = m.OptionDict.objects.get_or_create(options=def_opt)
        model, _ = m.OCRBoxModel.objects.get_or_create(**box)
        model.default_options = opt_obj
        model.entrypoint = entrypoint
        model.language_format = lcode
        model.iso1_map = iso1_map
        model.languages.clear()
        for l in lang:
            model.languages.add(m.Language.objects.get(iso1=l))
        model.save()

def auto_create_ocr():
    """Create OCRModel objects from entrypoints."""
    for ocr in load_ept_data('ocr_translate.ocr_data'):
        logger.debug(f'Creating ocr model: {ocr}')
        lang = ocr.pop('lang')
        lcode = ocr.pop('lang_code')
        entrypoint = ocr.pop('entrypoint')
        iso1_map = ocr.pop('iso1_map', {})
        def_opt = ocr.pop('default_options', {})
        opt_obj, _ = m.OptionDict.objects.get_or_create(options=def_opt)
        model, _ = m.OCRModel.objects.get_or_create(**ocr)
        model.default_options = opt_obj
        model.language_format = lcode
        model.iso1_map = iso1_map
        model.entrypoint = entrypoint
        model.languages.clear()
        for l in lang:
            model.languages.add(m.Language.objects.get(iso1=l))
        model.save()

def auto_create_tsl():
    """Create TSLModel objects from entrypoints."""
    for tsl in load_ept_data('ocr_translate.tsl_data'):
        logger.debug(f'Creating tsl model: {tsl}')
        src = tsl.pop('lang_src')
        dst = tsl.pop('lang_dst')
        lcode = tsl.pop('lang_code', None)
        entrypoint = tsl.pop('entrypoint')
        iso1_map = tsl.pop('iso1_map', {})
        def_opt = tsl.pop('default_options', {})
        opt_obj, _ = m.OptionDict.objects.get_or_create(options=def_opt)
        model, _ = m.TSLModel.objects.get_or_create(**tsl)
        model.default_options = opt_obj
        model.language_format = lcode
        model.iso1_map = iso1_map
        model.entrypoint = entrypoint
        model.src_languages.clear()
        for l in src:
            logger.debug(f'Adding src language: {l}')
            kwargs = {lcode: l}
            model.src_languages.add(*m.Language.objects.filter(**kwargs))

        model.dst_languages.clear()
        for l in dst:
            logger.debug(f'Adding dst language: {l}')
            kwargs = {lcode: l}
            model.dst_languages.add(*m.Language.objects.filter(**kwargs))
        model.save()

def auto_create_models():
    """Create OCR and TSL models from json file. Also create default OptionDict"""
    auto_create_box()
    auto_create_ocr()
    auto_create_tsl()

    m.OptionDict.objects.get_or_create(options={})



# Generated by Django 4.2.3 on 2023-07-30 18:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [('ocr_translate', '0002_ocrboxmodel_default_options_ocrmodel_default_options_and_more'), ('ocr_translate', '0003_language_default_options'), ('ocr_translate', '0004_remove_language_break_chars_and_more'), ('ocr_translate', '0005_alter_ocrboxmodel_default_options_and_more')]

    dependencies = [
        ('ocr_translate', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='language',
            name='default_options',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='lang_default_options', to='ocr_translate.optiondict'),
        ),
        migrations.RemoveField(
            model_name='language',
            name='break_chars',
        ),
        migrations.RemoveField(
            model_name='language',
            name='ignore_chars',
        ),
        migrations.AddField(
            model_name='ocrboxmodel',
            name='default_options',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='box_default_options', to='ocr_translate.optiondict'),
        ),
        migrations.AddField(
            model_name='ocrmodel',
            name='default_options',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ocr_default_options', to='ocr_translate.optiondict'),
        ),
        migrations.AddField(
            model_name='tslmodel',
            name='default_options',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tsl_default_options', to='ocr_translate.optiondict'),
        ),
    ]



# Generated by Django 4.2.2 on 2023-07-17 14:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BBox',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('l', models.IntegerField()),
                ('b', models.IntegerField()),
                ('r', models.IntegerField()),
                ('t', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('md5', models.CharField(max_length=32, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Language',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
                ('iso1', models.CharField(max_length=8, unique=True)),
                ('iso2b', models.CharField(max_length=8, unique=True)),
                ('iso2t', models.CharField(max_length=8, unique=True)),
                ('iso3', models.CharField(max_length=32, unique=True)),
                ('easyocr', models.CharField(max_length=32, null=True)),
                ('tesseract', models.CharField(max_length=32, null=True)),
                ('facebookM2M', models.CharField(max_length=32, null=True)),
                ('break_chars', models.CharField(max_length=512, null=True)),
                ('ignore_chars', models.CharField(max_length=512, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='OCRBoxModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                ('language_format', models.CharField(max_length=32, null=True)),
                ('languages', models.ManyToManyField(related_name='box_models', to='ocr_translate.language')),
            ],
        ),
        migrations.CreateModel(
            name='OCRModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                ('language_format', models.CharField(max_length=32, null=True)),
                ('languages', models.ManyToManyField(related_name='ocr_models', to='ocr_translate.language')),
            ],
        ),
        migrations.CreateModel(
            name='OptionDict',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('options', models.JSONField(unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Text',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='TSLModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                ('language_format', models.CharField(max_length=32, null=True)),
                ('dst_languages', models.ManyToManyField(related_name='tsl_models_dst', to='ocr_translate.language')),
                ('src_languages', models.ManyToManyField(related_name='tsl_models_src', to='ocr_translate.language')),
            ],
        ),
        migrations.CreateModel(
            name='TranslationRun',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lang_dst', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trans_dst', to='ocr_translate.language')),
                ('lang_src', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trans_src', to='ocr_translate.language')),
                ('model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tsl_runs', to='ocr_translate.tslmodel')),
                ('options', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trans_options', to='ocr_translate.optiondict')),
                ('result', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='from_trans', to='ocr_translate.text')),
                ('text', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='to_trans', to='ocr_translate.text')),
            ],
        ),
        migrations.CreateModel(
            name='OCRRun',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bbox', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='to_ocr', to='ocr_translate.bbox')),
                ('lang_src', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ocr_src', to='ocr_translate.language')),
                ('model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ocr_runs', to='ocr_translate.ocrmodel')),
                ('options', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ocr_options', to='ocr_translate.optiondict')),
                ('result', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='from_ocr', to='ocr_translate.text')),
            ],
        ),
        migrations.CreateModel(
            name='OCRBoxRun',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='to_box', to='ocr_translate.image')),
                ('lang_src', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='box_src', to='ocr_translate.language')),
                ('model', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='box_runs', to='ocr_translate.ocrboxmodel')),
                ('options', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ocr_box_options', to='ocr_translate.optiondict')),
            ],
        ),
        migrations.AddField(
            model_name='bbox',
            name='from_ocr',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='result', to='ocr_translate.ocrboxrun'),
        ),
        migrations.AddField(
            model_name='bbox',
            name='image',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='bboxes', to='ocr_translate.image'),
        ),
    ]



# Generated by Django 4.2.4 on 2023-08-03 13:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ocr_translate', '0002_ocrboxmodel_default_options_ocrmodel_default_options_and_more_squashed_0005_alter_ocrboxmodel_default_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='tslmodel',
            name='entrypoint',
            field=models.CharField(max_length=128, null=True),
        ),
    ]
